#include <iostream>
#include <Eigen/Dense>
#include <R.h>
#include <Rmath.h>
#include <time.h>
#include <sys/time.h>

using namespace Eigen;
using namespace std;

//g++ -o EigenExample EigenExample.cpp -I/usr/include/eigen3 -I/usr/share/R/include -lRmath -lR -O2 -Wall -fopenmp  -DEIGEN_NO_DEBUG
// onlyl use the -DEIGEN_NO_DEBUG once you've made sure your code works
// Eigen package (libeigen3 on Ubuntu) needs to be installed on your system

int main()
{
  //omp_set_num_threads(8); // ???
  timeval start, stop, result1, result2;

  int size = 8000;
  MatrixXd mat(size, size);

  for(int i = 0; i < size; i++){
    for(int j = 0; j < size; j++){
      mat(i,j) = rnorm(0.0, 1.0);
    }
  }
 
  cout << "done with rnorm" << endl;

  // form 1
  gettimeofday(&start, NULL);

  LLT<MatrixXd> chol1;
  // I believe this is threaded - seeing ~400% cpu usage, but it might only be the transpose and multiplication and not the Cholesky
  chol1.compute(mat.transpose() * mat);

  gettimeofday(&stop, NULL);
  timersub(&stop, &start, &result1);


  // form 2 from Dirk Eddelbuettel's JSS paper - I think this allows compiler to better optimize,
  // this appears not to be threaded - seeing 100% CPU usage and Eigen doc says something about only matrix stuff is threaded so maybe these calcs are not
  gettimeofday(&start, NULL);
 
  LLT<MatrixXd> chol2(MatrixXd(size,size).setZero().
                      selfadjointView<Lower>().rankUpdate(mat.adjoint()));
  
  gettimeofday(&stop, NULL);
  timersub(&stop, &start, &result2);

  cout << "done chol" << endl;
  cout << "timing for first approach: " << result1.tv_sec + result1.tv_usec/1000000.0 << endl; // 68 sec.
  cout << "timing for second approach: " << result2.tv_sec + result2.tv_usec/1000000.0 << endl; // 55 sec.
  // by comparison, in R chol(crossprod(x)), threaded (it is using ~400% CPU): 40 sec. (128 total across the threads)
  // by comparison, in R chol(crossprod(x)), using 1 thread: 83 sec.

}
