library(inline)
library(RcppEigen)

incl <- c("using namespace Eigen;")

src <- '
  const Map<MatrixXd> mat(as<Map<MatrixXd> >(x));
  LLT<MatrixXd> cholFactor(mat);
  return(wrap(MatrixXd(cholFactor.matrixU())));
'

cholE = cxxfunction(signature(x = "matrix"), src, plugin="RcppEigen", includes = incl, libargs="-fopenmp") 


n = 4000
x = crossprod(matrix(rnorm(n^2), n))

library(rbenchmark)
benchmark(
  out <- chol(x),
  out <- cholE(x),
           replications = 5, columns = c("test", "elapsed", "replications")
          )

# with a single thread these are comparable, but chol() is faster when threaded, because the Eigen cholesky appears not to be threaded
# out <- cholE(x)  15.953            5
#  out <- chol(x)  17.496            5

