library(inline)
library(Rcpp)

fun <- cxxfunction(, "", includes = '
double norm(NumericVector x, NumericVector y) {
double sum = 0.0;
for(int i = 0; i < x.size(); i++) {
sum += x[i]*x[i] + y[i]*y[i];
}
return(sum);
}
RCPP_MODULE(mymodule){
function("norm", &norm); 
}
', plugin = "Rcpp")

mymodule <- Module("mymodule", getDynLib(fun))
norm <- mymodule$norm

norm
args(norm)

n <- 100
set.seed(0)
u <- rnorm(n); v <- rnorm(n)
norm(u, v)

sum(u^2+v^2)

# here's how you do it if you want your function to have named arguments

fun <- cxxfunction(, "", includes = '
double norm(NumericVector x, NumericVector y) {
double sum = 0.0;
for(int i = 0; i < x.size(); i++) {
sum += x[i]*x[i] + y[i]*y[i];
}
return(sum);
}
RCPP_MODULE(mymodule){
function("norm", &norm, List::create(_["x"],_["y"]));
}
', plugin = "Rcpp")

mymodule_v2 <- Module("mymodule", getDynLib(fun))
norm_v2 <- mymodule_v2$norm

norm_v2(y = v, x = u)

norm_v2
args(norm_v2)


