library(inline)

src <- "
  for (int i = 0; i < *n; i++) {
    x[i] = exp(x[i]);
  }
"

sillyExp <- cfunction(signature(n = "integer", x = "numeric"), src, convention = ".C")

set.seed(0)
n <- as.integer(100)
x <- rnorm(n)
out <- sillyExp(n, x = x)$x

library(rbenchmark)

set.seed(0)
n <- as.integer(100000)
x <- rnorm(n)
out <- rep(0, n)

print(benchmark(
      { for(i in 1:n) out[i] <- exp(x[i]) },  # stupidly unvectorized
      out <- exp(x),                          # vectorized
      out <- sillyExp(n, x = x)$x,            # call to C
      replications = 10, columns = c("test", "elapsed", "replications")
))


