# NOTE: pkgExample/files contains R/cpp/header/Makevars files to be used in constructing the package - some of these R files have help info that devtools/roxygen2 will turn into help files

# the commands below work on arwen.berkeley.edu, and likely work, perhaps with a bit of modification on Macs

# start R from the pkgExample directory

rm(list = ls()) # doing this because some of the cjpUtil functions might be in my environment as they are loaded via my .Rprofile; get rid of them so workspace is clean for development

library(devtools)

create('cjpUtils')  # or use package.skeleton()

system("cp files/misc.R cjpUtils/R/.")
system("cp files/objects.R cjpUtils/R/.")
system("cp files/plot.R cjpUtils/R/.")
system("cp files/cjpUtils-package.r cjpUtils/R")  # 'dummy' R file with help info for the package as a whole

load_all("cjpUtils")

system("mkdir cjpUtils/data; cp files/*rda cjpUtils/data")
# add "LazyData: Yes" to DESCRIPTION file

### next note that there is some documentation already in the roxygen format in some of those R files 
### note also cjpUtils-package.r, which exists solely for documenting the entire package via roxygen
### compare the roxygen syntax to the formatting in the .Rd files in, say, Matrix (see Sec. 3.3.5 of the workshop notes)

document('cjpUtils') # create the documentation; export/import info in the .R files in roxygen format will be included in NAMESPACE

# look at NAMESPACE and DESCRIPTION

# modify DESCRIPTION to also include
# Imports: fields
# Suggests: maps
# (maps is used in the pmap2() example
# note discussion of loading vs. attaching and importing/exporting (see Sec. 3.3.3 of the workshop notes)

load_all("cjpUtils")

dev_help("R2.Rd")
dev_help("cjpUtils.Rd")

dev_example("R2.Rd")
dev_example("pmap2.Rd")

### let's also create an example test for our package

system('mkdir cjpUtils/inst')
system('mkdir cjpUtils/inst/tests')   # R extensions manuals says to use cjpUtils/tests not cjpUtils/inst/tests, but devtools wants it in inst/tests
system('cp files/testR2.R cjpUtils/inst/tests')


test('cjpUtils')

# try mucking up R2 (syntax error and run-time error) and see what happens when test it

load_all("cjpUtils")
test('cjpUtils')

### now let's try building, checking, installing

build('cjpUtils')  # R CMD build cjpUtils
check('cjpUtils')  # R CMD check cjpUtils_1.0.tar.gz

### I'm not totally sure where install() will install things or how that will coincide with the development version in our current R session, so let's exit and try an install separately back in UNIX

# let's mv /inst/tests to /tests - R's regular package stuff seems to want them there for testing
system("mv cjpUtils/inst/tests cjpUtils/.")

# also add the line 'library(cjpUtils)' at the start of testR2.R

q()

# here's how to build, check and install from the UNIX command line
# R CMD build cjpUtils
# R CMD check cjpUtils_1.0.tar.gz
# R CMD INSTALL -l /tmp cjpUtils_1.0.tar.gz

# back in new session of R

library('cjpUtils', lib.loc = '/tmp')
?cjpUtils
example('R2')
example('cjpUtils')

### now let's try adding some C++ code based on Rcpp skeleton

system('mkdir cjpUtils/src')

# see /usr/local/linux/lib/R/Current/x86_64/site-library/Rcpp/skeleton 

system('cp files/RcppExample.h cjpUtils/src')
system('cp files/RcppExample.cpp cjpUtils/src')
system('cp files/RcppExample.R cjpUtils/R')
system('cp files/Makevars cjpUtils/src')
system('cp files/Makevars.win cjpUtils/src')
# note (as of 5/22/13) -- we don't need the PKG_CPPFLAGS line in the Makevars files; instead put
# "LinkingTo: Rcpp" as a line in the DESCRIPTION file

# add dependency on Rcpp to DESCRIPTION
# add RcppExample.R to collates field in DESCRIPTION
# add 'useDynLib(cjpUtils)' and 'export(expC)' to NAMESPACE 

# now do building,checking,installing again
# R CMD build cjpUtils
# R CMD check cjpUtils_1.0.tar.gz
# R CMD INSTALL -l /tmp cjpUtils_1.0.tar.gz

# now start R and test the new function that uses Rcpp
library(cjpUtils, lib.loc = '/tmp')
expC(1:5) # the silly function that is an R wrapper to C++ code (via Rcpp) that exponentiates a vector
