#' Calculate test R^2 and RMSE for predictions
#'
#' Based on vector of predictions and a vector of test observations, this function calculates the R^2 and RMSE of the predictions.
#'
#' @export
#' @param vals vector of observations
#' @param preds vector of corresponding predictions
#' @return vector of two values: R^2 and RMSE
#' @examples
#' R2(rnorm(10), rnorm(10))
R2 <- function(vals, preds){
  return(c(1-sum((vals - preds)^2) /
           sum((vals - mean(vals))^2 ),
           sqrt(mean((vals - preds)^2))))
}
