library(inline)

src <- '
NumericVector vec(xin);  // NumericVector constructor
// same as NumericVector vec = xin;
// in both cases there is an implicit cast that works because xin is a standard R vector
for(int i=0; i<vec.size(); i++) {
vec[i] = exp(vec[i]);
}
return(vec);  // no need for wrap as we have an Robject already
'

notSoSillyExp <- cxxfunction(signature(xin="numeric"), src, plugin = "Rcpp")

set.seed(0)
n <- as.integer(10000000)
x <- rnorm(n)

library(rbenchmark)

print(benchmark(
  out <- exp(x),
  out <- notSoSillyExp(x),
          replications = 5, columns = c("test", "elapsed", "replications")
          ))

# Not so 'silly' after all!!! I'm a bit surprised - we can beat R's vectorized exp(), which is itself just a direct call to C

#                      test elapsed replications
# 1           out <- exp(x)   0.755            5
# 2 out <- notSoSillyExp(x)   0.608            5


src <- '
List mylist(inlist);
return(mylist);
'
tmp = cxxfunction(signature(inlist ="list"), src, plugin = "Rcpp")

tmp(list(a=7))
