library(inline)
library(RcppArmadillo)

src <- '
arma::mat m1 = as<arma::mat>(mx);
arma::mat matMult = m1 * m1;
arma::mat crProd = m1.t() * m1;
arma::mat matScalMult = m1 * 2.0;
return(List::create(matMult, crProd, matScalMult));'

fun <- cxxfunction(signature(mx = "numeric"), src, plugin="RcppArmadillo")

mat <- matrix(as.double(1:9), 3)

identical(fun(mat), list(mat %*% mat, crossprod(mat), mat * 2))

src <- '
arma::mat m1 = as<arma::mat>(mx);
return(wrap(m1.t() * m1));
'

crossprodA = cxxfunction(signature(mx = "numeric"), src, plugin="RcppArmadillo")

n = 3000
x = matrix(rnorm(n^2), n)

library(rbenchmark)
benchmark(
  out <- crossprod(x),
  out <- crossprodA(x),
  out <- t(x) %*% x,
           replications = 5, columns = c("test", "elapsed", "replications")
          )

# this test is (in some sense) unfair to Armadillo as it doesn't have a matrix crossproduct function, while R's crossprod() exploits simplifications relative to t(X)%*%X

# single thread:
#                  test elapsed replications
#2 out <- crossprodA(x)  23.890            5
#1  out <- crossprod(x)  11.990            5
#3    out <- t(x) %*% x  24.496            5

# 4 threads:
#                  test elapsed replications
#2 out <- crossprodA(x)   6.816            5
#1  out <- crossprod(x)   3.733            5
#3    out <- t(x) %*% x   7.384            5





